/* 
 * tap-voip.c
 *
 * Provides the -z voip,calls tap
 *
 */

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <locale.h>
#include <glib.h>

#include "epan/packet_info.h"
#include "epan/value_string.h"
#include <epan/tap.h>
#include <epan/stat_tap_ui.h>
#include <epan/addr_resolv.h>
#include "ui/voip_calls.h"
#include "ui/rtp_stream.h"
#include "epan/sequence_analysis.h"

/* HACKY HACKY 
 *
 * The cf_retap_packets call doesn't seem to be necessary
 * when doing VOIP stuff, so it's OK if it's a NOP, it shouldn't get called.
 *
 * ... I don't think.
 */
#include "file.h"
cf_read_status_t
cf_retap_packets(capture_file *cf)
{
    (void)cf;
    return CF_READ_OK;
}

#define VOIP_CONVS "voip,convs"
#define VOIP_CALLS "voip,calls"

/* store the chosen calls in a bit-array */
#define VOIP_CONV_BITS          (sizeof(int) * 8)
#define VOIP_CONV_NUM           ((1<<(sizeof(guint16) * 8))/VOIP_CONV_BITS)
#define VOIP_CONV_MAX           (VOIP_CONV_BITS * VOIP_CONV_NUM)

static voip_calls_tapinfo_t tapinfo_;
static int voip_conv_sel[VOIP_CONV_NUM];

static void voip_convs_tap_draw(void *tapinfo_ptr)
{
    voip_calls_tapinfo_t *tapinfo = (voip_calls_tapinfo_t *) tapinfo_ptr;
    seq_analysis_info_t *sainfo = tapinfo->graph_analysis;

    printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
            "Frame",
            "Call",
            "Time",
            "Dst Addr",
            "Dst Port",
            "Src Addr",
            "Src Port",
            "Label",
            "Comment");

    sequence_analysis_list_sort(sainfo);

    for (GList *cur = g_queue_peek_nth_link(sainfo->items, 0); cur; cur = g_list_next(cur)) {
        seq_analysis_item_t *sai = (seq_analysis_item_t *) cur->data;

        if ((voip_conv_sel[sai->conv_num / VOIP_CONV_BITS] & (1 << (sai->conv_num % VOIP_CONV_BITS))) == 0)
            continue;


        printf("%d\t%d\t%s\t%s\t%d\t%s\t%d\t%s\t%s\n",
                sai->frame_number,
                sai->conv_num,
                sai->time_str,
                (char*)address_to_display(NULL, &(sai->dst_addr)),
                sai->port_dst,
                (char*)address_to_display(NULL, &(sai->src_addr)),
                sai->port_src,
                sai->frame_label,
                sai->comment);
    }
}

static void voip_calls_tap_draw(void *tapinfo_ptr)
{
    voip_calls_tapinfo_t *tapinfo = (voip_calls_tapinfo_t *) tapinfo_ptr;
    GList *cur_call = g_queue_peek_nth_link(tapinfo->callsinfos, 0);

    printf("VoIP Calls:\n");
    printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
            "Call", 
            "Start Time",
            "Stop Time",
            "Initial Speaker",
            "From",
            "To",
            "Protocol",
            "Packets",
            "State",
            "Comment");

    while (cur_call && cur_call->data) {
        voip_calls_info_t *call_info_ = (voip_calls_info_t*) cur_call->data;

        printf("%" G_GUINT16_FORMAT "\t%.6f\t%.6f\t%s\t%s\t%s\t%s\t%" G_GUINT16_FORMAT "\t%s\t%s\n",
                call_info_->call_num,
                nstime_to_sec(&(call_info_->start_rel_ts)),
                nstime_to_sec(&(call_info_->stop_rel_ts)), 
                (char*)address_to_display(NULL, &(call_info_->initial_speaker)),
                call_info_->from_identity,
                call_info_->to_identity,
                ((call_info_->protocol == VOIP_COMMON) && call_info_->protocol_name) ? call_info_->protocol_name : voip_protocol_name[call_info_->protocol],
                call_info_->npackets,
                voip_call_state_name[call_info_->call_state],
                call_info_->call_comment);
                
        cur_call = g_list_next(cur_call);
    }

}

static void voip_stat_init_tapinfo(void)
{
    memset(&tapinfo_, 0, sizeof(tapinfo_));
    tapinfo_.callsinfos = g_queue_new();
    
    /* fs_option FLOW_ALL shows the same info as the "SIP Flows" Wireshark tool
     * FLOW_ONLY_INVITES shows the same thing as "VoIP Flows" in Wireshark.
     * not totally sure what this really means right now. I believe we want FLOW_ONLY_INVITES?
     * this matches the Wireshark menu options and shows fewer streams.
     */
    tapinfo_.fs_option = FLOW_ONLY_INVITES;

    // add graph analysis
    tapinfo_.graph_analysis = sequence_analysis_info_new();
    tapinfo_.graph_analysis->name = "voip";
}


static void voip_calls_stat_init(const char *opt_arg _U_, void *userdata _U_)
{
    GString             *err_p;

    voip_stat_init_tapinfo();

    // register my own tap listener in addition to using their 
    err_p = register_tap_listener("frame", &tapinfo_, NULL,
            0,
            NULL,
            NULL,
            voip_calls_tap_draw,
            NULL);

    if (err_p != NULL)
    {
        printf("error registereing tap:%s\n", err_p->str);
        g_string_free(err_p, TRUE);
        exit(1);
    }

    // init their taps
    voip_calls_init_all_taps(&tapinfo_);
}

static void voip_convs_stat_init(const char *opt_arg, void *userdata _U_)
{
    GString *err_p;
    int len;
    unsigned int min, max, num;

    // init our shared tap info
    voip_stat_init_tapinfo();

    // parse the opt_arg to get which call we are asking for
    opt_arg += strlen(VOIP_CONVS);
    if (*opt_arg == 0) {
        memset(voip_conv_sel, -1, sizeof voip_conv_sel);
    } else {

        while (*opt_arg != 0) {
            if (sscanf(opt_arg, ",%u-%u%n", &min, &max, &len) == 2) {
                if (min <= max && min < VOIP_CONV_MAX && max < VOIP_CONV_MAX) {
                    for(; min <= max; min++) {
                        voip_conv_sel[min / VOIP_CONV_BITS] |= 1 << (min % VOIP_CONV_BITS);
                    }
                } else {
                    printf("Invalid 'convs' number range.\n");
                    exit(1);
                }
                opt_arg += len;
            } else if (sscanf(opt_arg, ",%u%n", &num, &len) == 1) {
                if (num < VOIP_CONV_MAX) {
                    voip_conv_sel[num / VOIP_CONV_BITS] |= 1 << (num % VOIP_CONV_BITS);
                } else {
                    printf("Invalid 'convs' call number.\n");
                    exit(1);
                }
                opt_arg += len;
            } else {
                printf("Invalid 'convs' parameter.\n");
                exit(1);
            }
        }
    }


    // register my own tap listener in addition to using theirs
    err_p = register_tap_listener("frame", &tapinfo_, NULL,
            0,
            NULL,
            NULL,
            voip_convs_tap_draw,
            NULL);

    if (err_p != NULL)
    {
        printf("error registering tap:%s\n", err_p->str);
        g_string_free(err_p, TRUE);
        exit(1);
    }

    // init their tap
    voip_calls_init_all_taps(&tapinfo_);
}
    

static stat_tap_ui voip_calls_ui = {
    REGISTER_STAT_GROUP_GENERIC,
    NULL,
    VOIP_CALLS,
    voip_calls_stat_init,
    0,
    NULL
};

/* Register this tap listener (need void on own so line register function found) */
void
register_tap_listener_voip_calls(void)
{
    register_stat_tap_ui(&voip_calls_ui, NULL);
}


static stat_tap_ui voip_convs_ui = {
    REGISTER_STAT_GROUP_GENERIC,
    NULL,
    VOIP_CONVS,
    voip_convs_stat_init,
    0,
    NULL
};

/* Register this tap listener (need void on own so line register function found) */
void
register_tap_listener_voip_convs(void)
{
    register_stat_tap_ui(&voip_convs_ui, NULL);
}



/*
 * Editor modelines  -  http://www.wireshark.org/tools/modelines.html
 *
 * Local variables:
 * c-basic-offset: 4
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 *
 * vi: set shiftwidth=4 tabstop=8 expandtab:
 * :indentSize=4:tabSize=8:noTabs=true:
 *
 */

